/* ***** BEGIN LICENSE BLOCK Version: GPL 3.0 ***** 
 * FireMobileFimulator is a Firefox add-on that simulate web browsers of 
 * japanese mobile phones.
 * Copyright (C) 2008  ryster <ryster@php-develop.org>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * ***** END LICENSE BLOCK ***** */

var firemobilesimulator;
if(!firemobilesimulator) firemobilesimulator = {};
if(!firemobilesimulator.mpc) firemobilesimulator.mpc = {};

firemobilesimulator.mpc.ezweb = function(charset) {
	this.charset = charset || firemobilesimulator.mpc.common.MPC_SJIS;
};
firemobilesimulator.mpc.ezweb.prototype = {
	/**
	 * EZweb絵文字画像格納パス
	 * 
	 * @var string
	 */
	e_img_path : "img/e/",

	iconMap : null,

	charset : null,

	/**
	 * 文字列からEZweb絵文字を検出し、指定されたフォーマットに変換 基本・拡張・隠し絵文字対応
	 * 
	 * @return string
	 */
	_____convert : function(str) {
		// 2008/09/03 この変換は正規表現でなくてDOMでやった方が良いので使っていない
		var re1 = /<img\s+(?:icon|localsrc)\s*=\s*(["']?)([0-9]+)\1\s*>/ig;

		var _this = this;
		var f = function(whole, s1, s2) {
			return _this.e_options_encode(s2);
		};
		str = str.replace(re1, f);
		return str;
	},

	/**
	 * 絵文字画像格納ディレクトリの一括設定
	 * 
	 * @param string
	 *            path
	 */
	setImagePath : function(path) {
		path.replace(/\/+$/, '');
		this.e_img_path = path + '/e/';
	},

	/**
	 * EZweb絵文字（icon番号）を指定されたimgタグ形式へ変換
	 * 
	 * @param integer
	 *            iconno
	 * @return string
	 */
	e_options_encode : function(iconno) {
		iconno = parseInt(iconno, 10);
		var width = (iconno == 174) ? 7 : ((iconno == 175) ? 4 : 15);
		var buf = '<img src="' + this.getImageSrc(iconno)
				+ '" alt="" border="0" width="' + width + '" height="15" />';
		return buf;
	},

	getImageSrc : function(iconno) {
		return this.e_img_path.replace(/\/+$/, "") + '/' + iconno + '.gif';
	},

	getIconNumFromIconName : function(iconname){
		//javascriptの予約語が含まれているため、キー名の先頭に_(アンダーバー)を付与している
		var map = {
			_exclamation1 : 1,
			_exclamation2 : 2,
			_question1 : 3,
			_question2 : 4,
			_lefttri1 : 5,
			_righttri1 : 6,
			_lefttri2 : 7,
			_righttri2 : 8,
			_littlesquare1 : 9,
			_littlesquare2 : 10,
			_isymbol : 11,
			_wineglass : 12,
			_speaker : 13,
			_dollarsign : 14,
			_moon1 : 15,
			_bolt : 16,
			_medsquare1 : 17,
			_medsquare2 : 18,
			_littlediamond1 : 19,
			_littlediamond2 : 20,
			_bigsquare1 : 21,
			_bigsquare2 : 22,
			_littlecircle1 : 23,
			_littlecircle2 : 24,
			_wristwatch : 25,
			_plus : 26,
			_minus : 27,
			_star1 : 28,
			_uparrow1 : 29,
			_downarrow1 : 30,
			_circleslash : 31,
			_downtri1 : 32,
			_uptri1 : 33,
			_downtri2 : 34,
			_uptri2 : 35,
			_bigdiamond1 : 36,
			_bigdiamond2 : 37,
			_biggestsquare1 : 38,
			_biggestsquare2 : 39,
			_bigcircle1 : 40,
			_bigcircle2 : 41,
			_uparrow2 : 42,
			_downarrow2 : 43,
			_sun : 44,
			_baseball : 45,
			_clock : 46,
			_moon2 : 47,
			_bell : 48,
			_pushpin : 49,
			_smallface : 50,
			_heart : 51,
			_martini : 52,
			_bud : 53,
			_trademark : 54,
			_multiply : 55,
			_document1 : 56,
			_hourglass1 : 57,
			_hourglass2 : 58,
			_floppy1 : 59,
			_snowflake : 60,
			_cross1 : 61,
			_cross2 : 62,
			_rightarrow1 : 63,
			_leftarrow1 : 64,
			_mug : 65,
			_divide : 66,
			_calendar : 67,
			_smileyface : 68,
			_star2 : 69,
			_rightarrow2 : 70,
			_leftarrow2 : 71,
			_gem : 72,
			_checkmark1 : 73,
			_dog : 74,
			_star3 : 75,
			_sparkle : 76,
			_lightbulb : 77,
			_bird : 78,
			_folder1 : 79,
			_head1 : 80,
			_copyright : 81,
			_registered : 82,
			_briefcase : 83,
			_folder2 : 84,
			_phone1 : 85,
			_voiceballoon : 86,
			_creditcard : 87,
			_uptri3 : 88,
			_downtri3 : 89,
			_usa : 90,
			_note3 : 91,
			_clipboard : 92,
			_cup : 93,
			_camera1 : 94,
			_rain : 95,
			_football : 96,
			_book1 : 97,
			_stopsign : 98,
			_trafficlight : 99,
			_book2 : 100,
			_book3 : 101,
			_book4 : 102,
			_document2 : 103,
			_scissors : 104,
			_day : 105,
			_ticket : 106,
			_cloud : 107,
			_envelope1 : 108,
			_check : 109,
			_videocam : 110,
			_camcorder : 111,
			_house : 112,
			_flower : 113,
			_knife : 114,
			_vidtape : 115,
			_glasses : 116,
			_roundarrow1 : 117,
			_roundarrow2 : 118,
			_magnifyglass : 119,
			_key : 120,
			_note1 : 121,
			_note2 : 122,
			_boltnut : 123,
			_shoe : 124,
			_car : 125,
			_floppy2 : 126,
			_chart : 127,
			_graph1 : 128,
			_mailbox : 129,
			_flashlight : 130,
			_rolocard : 131,
			_check2 : 132,
			_leaf : 133,
			_hound : 134,
			_battery : 135,
			_scroll : 136,
			_thumbtack : 137,
			_lockkey : 138,
			_dollar : 139,
			_lefthand : 140,
			_righthand : 141,
			_tablet : 142,
			_paperclip : 143,
			_present : 144,
			_tag : 145,
			_meal1 : 146,
			_books : 147,
			_truck : 148,
			_pencil : 149,
			_uplogo : 150,
			_envelope2 : 151,
			_wrench : 152,
			_outbox : 153,
			_inbox : 154,
			_phone2 : 155,
			_factory : 156,
			_ruler1 : 157,
			_ruler2 : 158,
			_graph2 : 159,
			_meal2 : 160,
			_phone3 : 161,
			_plug : 162,
			_family : 163,
			_link : 164,
			_package : 165,
			_fax : 166,
			_partcloudy : 167,
			_plane : 168,
			_boat : 169,
			_dice : 170,
			_newspaper : 171,
			_train : 172,
			_blankfull : 173,
			_blankhalf : 174,
			_blankquarter : 175,
			_smoking : 176,
			_nosmoking : 177,
			_wheelchair : 178,
			_youngleaf : 179,
			_1square : 180,
			_2square : 181,
			_3square : 182,
			_4square : 183,
			_5square : 184,
			_6square : 185,
			_7square : 186,
			_8square : 187,
			_9square : 188,
			_10square : 189,
			_typhoon : 190,
			_snowman : 191,
			_aries : 192,
			_taurus : 193,
			_gemini : 194,
			_cancer : 195,
			_leo : 196,
			_virgo : 197,
			_libra : 198,
			_scorpio : 199,
			_sagittarius : 200,
			_capricorn : 201,
			_aquarius : 202,
			_pisces : 203,
			_ophiuchus : 204,
			_atm : 205,
			_24hours : 206,
			_toilet : 207,
			_parking : 208,
			_busstop : 209,
			_antenna : 210,
			_harbor : 211,
			_bank : 212,
			_gasstation : 213,
			_map : 214,
			_bicycle : 215,
			_bus : 216,
			_superexpress : 217,
			_marathon : 218,
			_soccer : 219,
			_tennis : 220,
			_snowboard : 221,
			_checkerflag : 222,
			_amusementpark : 223,
			_hotspring : 224,
			_bistro : 225,
			_movie : 226,
			_nightbridge : 227,
			_tower : 228,
			_slot : 229,
			_decoratedball : 230,
			_dart : 231,
			_controller : 232,
			_win : 233,
			_christmastree : 234,
			_cherryblossom : 235,
			_ghost : 236,
			_flag : 237,
			_watermelon : 238,
			_cake : 239,
			_pan : 240,
			_cherry : 241,
			_globefish : 242,
			_strawberry : 243,
			_riceball : 244,
			_hamburger : 245,
			_whale : 246,
			_rabbit : 247,
			_horse : 248,
			_monkey : 249,
			_frog : 250,
			_cat : 251,
			_penguin : 252,
			_ant : 253,
			_pig : 254,
			_coconut : 255,
			_sunflower : 256,
			_face1 : 257,
			_face2 : 258,
			_face3 : 259,
			_face4 : 260,
			_sleepy : 261,
			_anger : 262,
			_drip : 263,
			_kidding : 264,
			_heart2 : 265,
			_heart3 : 266,
			_sparkle2 : 267,
			_bomb : 268,
			_fire : 269,
			_sos : 270,
			_power : 271,
			_heart5 : 272,
			_kissmark : 273,
			_alien : 274,
			_fishcake : 275,
			_footmark : 276,
			_devil : 277,
			_flowercircle : 278,
			_secretsign : 279,
			_fullmarks : 280,
			_punch : 281,
			_dash : 282,
			_bigjob : 283,
			_firstfingersign : 284,
			_profitsign : 285,
			_skeelton : 286,
			_thumbsign : 287,
			_tv : 288,
			_microphone : 289,
			_wallet : 290,
			_melody : 291,
			_guitar : 292,
			_violin : 293,
			_headphone : 294,
			_rouge : 295,
			_action : 296,
			_estheticsalon : 297,
			_ezmark : 298,
			_freesign : 299,
			_cd : 300,
			_ladiesfashion : 301,
			_ufo : 302,
			_up : 303,
			_syringe : 304,
			_mist : 305,
			_golf : 306,
			_basketball : 307,
			_pager : 308,
			_art : 309,
			_drama : 310,
			_event : 311,
			_ribbon : 312,
			_birthday : 313,
			_spade : 314,
			_diamond : 315,
			_clover : 316,
			_eye : 317,
			_ear : 318,
			_peacesign : 319,
			_openfinger : 320,
			_moon3 : 321,
			_moon4 : 322,
			_moon5 : 323,
			_clear : 324,
			_0square : 325,
			_okay : 326,
			_face5 : 327,
			_lovely : 328,
			_impact : 329,
			_sweat : 330,
			_ezplus : 331,
			_earth : 332,
			_noodle : 333,
			_new : 334,
			_mensfashion : 335,
			_mensshoe : 336,
			_pc : 337,
			_radio2 : 338,
			_rose : 339,
			_chapel : 340,
			_subway : 341,
			_fuji : 342,
			_melody2 : 343,
			_angel : 344,
			_tiger : 345,
			_bear : 346,
			_rat : 347,
			_wink : 348,
			_face6 : 349,
			_shock : 350,
			_face7 : 351,
			_octopus : 352,
			_rocket : 353,
			_crown : 354,
			_kiss : 355,
			_hammer : 356,
			_fireworks : 357,
			_deadleaf : 358,
			_bag : 359,
			_fountain : 360,
			_camp : 361,
			_mahjongg : 362,
			_vs : 363,
			_trophy : 364,
			_turtle : 365,
			_spain : 366,
			_russia : 367,
			_construction : 368,
			_bath : 369,
			_flag2 : 370,
			_town : 371,
			_egg : 372,
			_stock : 373,
			_policeman : 374,
			_postoffice : 375,
			_hospital : 376,
			_school : 377,
			_hotel : 378,
			_ship : 379,
			_prohibition : 380,
			_within : 381,
			_cool : 382,
			_mark1 : 383,
			_mark2 : 384,
			_mark3 : 385,
			_mark4 : 386,
			_mark5 : 387,
			_mark6 : 388,
			_mark7 : 389,
			_uphand : 390,
			_underhand : 391,
			_fortunetelling : 392,
			_manners : 393,
			_off : 394,
			_memo : 395,
			_tie : 396,
			_hibiscus : 397,
			_flowers : 398,
			_cactus : 399,
			_sakebottle : 400,
			_toast : 401,
			_mark8 : 402,
			_medicine : 403,
			_balloon : 404,
			_cracker : 405,
			_eznavigation : 406,
			_hat : 407,
			_boots : 408,
			_nail : 409,
			_beautyparlor : 410,
			_barbershop : 411,
			_kimono : 412,
			_bikini : 413,
			_heart6 : 414,
			_heart7 : 415,
			_blueheart : 416,
			_greenheart : 417,
			_yellowheart : 418,
			_purpleheart : 419,
			_shine : 420,
			_skis : 421,
			_circle : 422,
			_japanesetea : 423,
			_bread : 424,
			_softcream : 425,
			_potato : 426,
			_dumpling : 427,
			_ricecracker : 428,
			_rice : 429,
			_spaghetti : 430,
			_curry : 431,
			_oden : 432,
			_sushi : 433,
			_apple : 434,
			_orange : 435,
			_tomato : 436,
			_eggplant : 437,
			_boxlunch : 438,
			_pot : 439,
			_face8 : 440,
			_face9 : 441,
			_face10 : 442,
			_face11 : 443,
			_face12 : 444,
			_face13 : 445,
			_face14 : 446,
			_face15 : 447,
			_face16 : 448,
			_face17 : 449,
			_face18 : 450,
			_face19 : 451,
			_face20 : 452,
			_melody3 : 453,
			_face21 : 454,
			_face22 : 455,
			_face23 : 456,
			_nose : 457,
			_mouth : 458,
			_face24 : 459,
			_clapping : 460,
			_ok : 461,
			_booing : 462,
			_bye : 463,
			_ng : 464,
			_ok2 : 465,
			_apologize : 466,
			_couple : 467,
			_bunnygirl : 468,
			_trumpet : 469,
			_billiards : 470,
			_swimming : 471,
			_fireengine : 472,
			_ambulance : 473,
			_policecar : 474,
			_rollercoaster : 475,
			_pinedecoration : 476,
			_dollsfestival : 477,
			_commencement : 478,
			_entranceceremony : 479,
			_carpstreamer : 480,
			_umbrella : 481,
			_bride : 482,
			_shavedice : 483,
			_toyfireworks : 484,
			_shell : 485,
			_windbell : 486,
			_halloween : 487,
			_moonlight : 488,
			_santaclaus : 489,
			_night : 490,
			_rainbow : 491,
			_hotel2 : 492,
			_daybreak : 493,
			_silkhat : 494,
			_departmentstore : 495,
			_castle : 496,
			_castle2 : 497,
			_factory2 : 498,
			_france : 499,
			_pclogo : 500,
			_lockcertificate : 501,
			_caps : 502,
			_lower : 503,
			_numbers : 504,
			_symbols : 505,
			_accept : 506,
			_checkbox : 507,
			_edit : 508,
			_radio : 509,
			_view : 510,
			_back : 511,
			_bookmarks : 512,
			_exit : 513,
			_home : 514,
			_inbox1 : 515,
			_mark : 516,
			_personal : 517,
			_reload : 518,
			_germany : 700,
			_italy : 701,
			_england : 702,
			_china : 703,
			_korea : 704,
			_whiterace : 705,
			_chinese : 706,
			_indian : 707,
			_grandfather : 708,
			_grandmother : 709,
			_baby : 710,
			_contractor : 711,
			_princess : 712,
			_dolphin : 713,
			_dance : 714,
			_tropicalfish : 715,
			_millipede : 716,
			_elephant : 717,
			_koala : 718,
			_cow : 719,
			_snake : 720,
			_fowl : 721,
			_wildboar : 722,
			_camel : 723,
			_blooda : 724,
			_bloodb : 725,
			_bloodo : 726,
			_bloodab : 727,
			_footprint : 728,
			_sneakers : 729,
			_flag3 : 730,
			_uparrow3 : 731,
			_downarrow3 : 732,
			_sign2 : 733,
			_sign3 : 734,
			_sign4 : 735,
			_melon : 736,
			_pineapple : 737,
			_grapes : 738,
			_banana : 739,
			_corn : 740,
			_mushroom : 741,
			_chestnut : 742,
			_peach : 743,
			_sweetpotato : 744,
			_pizza : 745,
			_turkey : 746,
			_starfestival : 747,
			_drink : 748,
			_dragon : 749,
			_piano : 750,
			_surfing : 751,
			_fishing : 752,
			_bowling : 753,
			_namahage : 754,
			_tengu : 755,
			_panda : 756,
			_face25 : 757,
			_pig2 : 758,
			_flower2 : 759,
			_icecream : 760,
			_doughnut : 761,
			_cookie : 762,
			_chocolate : 763,
			_candy : 764,
			_candy2 : 765,
			_see : 766,
			_say : 767,
			_hear : 768,
			_volcano : 769,
			_heart4 : 770,
			_abc : 771,
			_pudding : 772,
			_bee : 773,
			_ladybug : 774,
			_pot2 : 775,
			_greenapple : 776,
			_bill : 777,
			_sign5 : 778,
			_face26 : 779,
			_catface : 780,
			_milkyway : 781,
			_catface2 : 782,
			_catface3 : 783,
			_envelope4 : 784,
			_catface4 : 785,
			_face27 : 786,
			_catface5 : 787,
			_catface6 : 788,
			_face28 : 789,
			_catface7 : 790,
			_face29 : 791,
			_catface9 : 792,
			_dress : 793,
			_figure : 794,
			_station : 795,
			_japaneseplayingcards : 796,
			_joker : 797,
			_lobster : 798,
			_icon : 799,
			_walk : 800,
			_policecar2 : 801,
			_ezmovie : 802,
			_heart8 : 803,
			_chick : 804,
			_jeans : 805,
			_envelope3 : 806,
			_arrow : 807,
			_doubleheadedarrow : 808,
			_doubleheadedarrow2 : 809,
			_wave : 810,
			_bud2 : 811,
			_snail : 812,
			_catface8 : 813,
			_face31 : 814,
			_icon2 : 815,
			_herb : 816,
			_closefinger : 817,
			_sharp : 818,
			_raisehand : 819,
			_character : 820,
			_character2 : 821,
			_character3 : 822,
			_ezapplij : 823,
			_ezapplib : 824,
			_ezchakuuta : 825,
			_eznavims : 826,
			_winmark : 827,
			_premiummark : 828
		};
		return map[iconname];
	},

	/**
	 * 絵文字のKDDI公式SJISコードと比較して、bit列が絵文字かどうかを判定する
	 * 
	 * @param {}
	 *            chs
	 * @return {}
	 */
	isPictogramSJISDecs : function(chs) {
		var result;
		if (chs.length != 2) {
			return false;
		}

		[char1, char2] = chs;
		if (((char1 == 0xF3 || char1 == 0xF6 || char1 == 0xF7) && ((char2 >= 0x40 && char2 <= 0x7E) || (char2 >= 0x80 && char2 <= 0xFC)))
				|| (char1 == 0xF4 && ((char2 >= 0x40 && char2 <= 0x7E) || (char2 >= 0x80 && char2 <= 0x8D)))) {
			result = true;
		} else {
			result = false;
		}
		return result;
	},

	/**
	 * 絵文字の裏KDDI UnicodeのUTF8エンコーディングと比較して、bit列が絵文字がどうかを判定する。
	 * 
	 * @param {}
	 *            chs
	 * @return {Boolean}
	 */
	isPictogramUTF8Decs : function(chs) {
		var result;
		if (chs.length != 3) {
			return false;
		}

		// dump("isPictogram"+chs.join(":")+"");
		[char1, char2, char3] = chs;
		if ((char1 == 0xEE && ((char2 == 0xB1 && (char3 >= 0x80 && char3 <= 0xBE))
				|| (char2 == 0xB2 && (char3 >= 0x80 && char3 <= 0xBF))
				|| (char2 == 0xB3 && (char3 >= 0x80 && char3 <= 0xBC))
				|| (char2 == 0xB5 && (char3 >= 0x80 && char3 <= 0xBE))
				|| (char2 == 0xB6 && (char3 >= 0x80 && char3 <= 0x8D))
				|| (char2 == 0xBD && (char3 >= 0x80 && char3 <= 0xBE))
				|| (char2 == 0xBE && (char3 >= 0x80 && char3 <= 0xBF)) || (char2 == 0xBF && (char3 >= 0x80 && char3 <= 0xBC))))
				|| (char1 == 0xEF && ((char2 == 0x81 && (char3 >= 0x80 && char3 <= 0xBE))
						|| (char2 == 0x82 && (char3 >= 0x80 && char3 <= 0xBF)) || (char2 == 0x83 && (char3 >= 0x80 && char3 <= 0xBC))))) {
			result = true;
		} else {
			result = false;
		}

		if (char1 == 0xEE && char2 == 0x94) {
			dump("##probably pictogram?? "+char1+":"+char2+":"+char3+"\n")
		}
		// dump("=>"+result+"\n");
		return result;
	},

	/**
	 * 絵文字のKDDI公式Unicodeと比較して、bit列が絵文字かどうかを判定する
	 * 
	 * @param {}
	 *            chs
	 * @return {}
	 */
	isPictogramUnicodeDecs : function(chs) {
		var result;
		if (chs.length != 2) {
			return false;
		}

		// dump("isPictogram"+chs.join(":")+"");
		[char1, char2] = chs;
		if ((char1 == 0xE4 && (char2 >= 0x68 && char2 <= 0xFF))
				|| (char1 == 0xE5 && (char2 >= 0x00 && char2 <= 0xDF))
				|| (char1 == 0xEA && (char2 >= 0x80 && char2 <= 0xFF))
				|| (char1 == 0xEB && (char2 >= 0x00 && char2 <= 0x88))) {
			result = true;
		} else {
			result = false;
		}
		// dump("=>"+result+"\n");
		return result;
	},

	convert : function(str) {
		dump("[mpc]Au convert start.charset = " + this.charset + "\n");

		var _this = this;

		// TODO: Auは、Unicodeの数値参照とSJISの数値参照が混在する可能性があるので注意が必要
		// 16進数値参照を、SJIS文字コードに変換
		var re1 = /\&\#x([a-f0-9]{2})([a-f0-9]{2});/ig;
		str = str.replace(re1, function(whole, s1, s2) {
			dump("[mpc]au regmatch16:" + s1 + ":" + s2 + "\n");
			var dec1 = parseInt(s1, 16);
			var dec2 = parseInt(s2, 16);
			var decs = [dec1, dec2];
			if (_this.isPictogramSJISDecs(decs)) {
				// decsはSJISの文字コード
				dump("au SJIS 16 match\n");
				return _this
						.e_options_encode(_this.getIconMapSJIS()[firemobilesimulator.mpc.common.bits2dec(decs)]);
			} else if (_this.isPictogramUnicodeDecs(decs)) {
				// decsはUnicode
				dump("au Unicode 16 match\n");
				return _this
						.e_options_encode(_this.getIconMapUnicode()[firemobilesimulator.mpc.common.bits2dec(decs)]);
			} else {
				dump("au 16 no match\n")
				// return String.fromCharCode(dec1) + String.fromCharCode(dec2);
				return whole;
			}
		});
		// SJIS10進数値参照を、SJIS文字コードに変換
		var re2 = /\&\#([0-9]{5});/g;
		str = str.replace(re2, function(whole, s1) {
			dump("[mpc]au regmatch10:" + s1 + "\n");
			var bin;
			var sdec = parseInt(s1, 10);
			var decs;
			if (sdec >= 256) {
				var dec1 = parseInt(sdec / 256);
				var dec2 = sdec % 256;
				decs = [dec1, dec2];
				// bin = String.fromCharCode(dec1)
				// + String.fromCharCode(dec2);
			}
			if (_this.isPictogramSJISDecs(decs)) {
				// decsはSJISの文字コード
				dump("au SJIS 10 match\n");
				return _this
						.e_options_encode(_this.getIconMapSJIS()[firemobilesimulator.mpc.common.bits2dec(decs)]);
			} else if (_this.isPictogramUnicodeDecs(decs)) {
				// decsはUnicode
				dump("au Unicode 10 match\n");
				return _this
						.e_options_encode(_this.getIconMapUnicode()[firemobilesimulator.mpc.common.bits2dec(decs)]);
			} else {
				dump("au 10 no match\n");
				// return String.fromCharCode(dec1) + String.fromCharCode(dec2);
				return whole;
			}
		});

		// var hexstrings = unpack_e(str);
		var r = "";
		var hexstrings = new firemobilesimulator.mpc.common.HexStrings(firemobilesimulator.mpc.common.unpack(str), this.charset);

		dump("[mpc]Au binary match start\n");
		while (hexstrings.hasNextCharacter()) {
			var decs = hexstrings.getNextCharacterDecs();
			// 絵文字変換処理
			if (this.charset == firemobilesimulator.mpc.common.MPC_SJIS) {
				// SJISバイナリの絵文字を変換
				if (this.isPictogramSJISDecs(decs)) {
					r += this
							.e_options_encode(_this.getIconMapSJIS()[firemobilesimulator.mpc.common.bits2dec(decs)]);
				} else {
					for (var i = 0; i < decs.length; i++) {
						r += String.fromCharCode(decs[i]);
					}
				}
			} else if (this.charset == firemobilesimulator.mpc.common.MPC_UTF8) {
				// 裏KDDIのUTF-8バイナリの絵文字を変換
				if (this.isPictogramUTF8Decs(decs)) {
					r += this
							.e_options_encode(_this.getIconMapSJIS()[firemobilesimulator.mpc.common.bits2dec(_this
									.e_unofficialUnicode2SJIS(firemobilesimulator.mpc.common.utf82unicode(decs)))]);
				} else {
					for (var i = 0; i < decs.length; i++) {
						r += String.fromCharCode(decs[i]);
					}
				}
			} else {
				dump("[mpc]Au Unknown charset [" + this.charset + "].\n");
				return str;
			}
		}
		return r;
	},

	e_unofficialUnicode2SJIS : function(bits) {
		var r = new Array(2);
		if (bits.length != 2) {
			return r;
		}
		r[0] = bits[0] + 0x07;
		r[1] = bits[1];
		return r;
	},

	getIconMapSJIS : function() {
		if (!this.iconMap) {
			this.icomMap = {
				63065 : 1,
				63066 : 2,
				63067 : 3,
				63304 : 4,
				63305 : 5,
				63306 : 6,
				63307 : 7,
				63308 : 8,
				63309 : 9,
				63310 : 10,
				63311 : 11,
				63130 : 12,
				63210 : 13,
				63382 : 14,
				63070 : 15,
				63071 : 16,
				63312 : 17,
				63313 : 18,
				63314 : 19,
				63315 : 20,
				63316 : 21,
				63317 : 22,
				63318 : 23,
				63319 : 24,
				63383 : 25,
				63320 : 26,
				63321 : 27,
				63322 : 28,
				63323 : 29,
				63324 : 30,
				63325 : 31,
				63326 : 32,
				63327 : 33,
				63328 : 34,
				63329 : 35,
				63330 : 36,
				63331 : 37,
				63332 : 38,
				63333 : 39,
				63334 : 40,
				63335 : 41,
				63336 : 42,
				63337 : 43,
				63072 : 44,
				63123 : 45,
				63409 : 46,
				63073 : 47,
				63211 : 48,
				63356 : 49,
				63187 : 50,
				63410 : 51,
				63131 : 52,
				63212 : 53,
				63338 : 54,
				63339 : 55,
				63357 : 56,
				63384 : 57,
				63060 : 58,
				63358 : 59,
				63074 : 60,
				63340 : 61,
				63341 : 62,
				63342 : 63,
				63343 : 64,
				63132 : 65,
				63344 : 66,
				63360 : 67,
				63188 : 68,
				63075 : 69,
				63345 : 70,
				63346 : 71,
				63213 : 72,
				63347 : 73,
				63160 : 74,
				63040 : 75,
				63044 : 76,
				63054 : 77,
				63161 : 78,
				63404 : 79,
				63189 : 80,
				63348 : 81,
				63349 : 82,
				63092 : 83,
				63405 : 84,
				63411 : 85,
				63190 : 86,
				63385 : 87,
				63350 : 88,
				63351 : 89,
				63376 : 90,
				63093 : 91,
				63361 : 92,
				63412 : 93,
				63214 : 94,
				63076 : 95,
				63124 : 96,
				63362 : 97,
				63068 : 98,
				63042 : 99,
				63363 : 100,
				63364 : 101,
				63365 : 102,
				63366 : 103,
				63215 : 104,
				63367 : 105,
				63094 : 106,
				63077 : 107,
				63226 : 108,
				63386 : 109,
				63216 : 110,
				63387 : 111,
				63108 : 112,
				63165 : 113,
				63388 : 114,
				63389 : 115,
				63191 : 116,
				63352 : 117,
				63353 : 118,
				63217 : 119,
				63218 : 120,
				63368 : 121,
				63095 : 122,
				63390 : 123,
				63219 : 124,
				63114 : 125,
				63391 : 126,
				63377 : 127,
				63378 : 128,
				63220 : 129,
				63392 : 130,
				63369 : 131,
				63354 : 132,
				63143 : 133,
				63162 : 134,
				63393 : 135,
				63355 : 136,
				63370 : 137,
				63221 : 138,
				63394 : 139,
				63192 : 140,
				63193 : 141,
				63371 : 142,
				63096 : 143,
				63144 : 144,
				63222 : 145,
				63109 : 146,
				63372 : 147,
				63115 : 148,
				63097 : 149,
				63395 : 150,
				63406 : 151,
				63396 : 152,
				63407 : 153,
				63408 : 154,
				63223 : 155,
				63110 : 156,
				63373 : 157,
				63098 : 158,
				63379 : 159,
				63133 : 160,
				63397 : 161,
				63398 : 162,
				63194 : 163,
				63399 : 164,
				63224 : 165,
				63225 : 166,
				63078 : 167,
				63116 : 168,
				63117 : 169,
				63137 : 170,
				63400 : 171,
				63118 : 172,
				63401 : 173,
				63402 : 174,
				63403 : 175,
				63061 : 176,
				63062 : 177,
				63063 : 178,
				63064 : 179,
				63227 : 180,
				63228 : 181,
				63296 : 182,
				63297 : 183,
				63298 : 184,
				63299 : 185,
				63300 : 186,
				63301 : 187,
				63302 : 188,
				63303 : 189,
				63041 : 190,
				63069 : 191,
				63079 : 192,
				63080 : 193,
				63081 : 194,
				63082 : 195,
				63083 : 196,
				63084 : 197,
				63085 : 198,
				63086 : 199,
				63087 : 200,
				63088 : 201,
				63089 : 202,
				63090 : 203,
				63091 : 204,
				63099 : 205,
				63100 : 206,
				63101 : 207,
				63102 : 208,
				63104 : 209,
				63105 : 210,
				63106 : 211,
				63107 : 212,
				63374 : 213,
				63375 : 214,
				63111 : 215,
				63112 : 216,
				63113 : 217,
				63043 : 218,
				63119 : 219,
				63120 : 220,
				63121 : 221,
				63122 : 222,
				63045 : 223,
				63125 : 224,
				63126 : 225,
				63127 : 226,
				63128 : 227,
				63129 : 228,
				63046 : 229,
				63047 : 230,
				63134 : 231,
				63135 : 232,
				63136 : 233,
				63138 : 234,
				63139 : 235,
				63140 : 236,
				63141 : 237,
				63142 : 238,
				63145 : 239,
				63146 : 240,
				63147 : 241,
				63148 : 242,
				63149 : 243,
				63150 : 244,
				63151 : 245,
				63048 : 246,
				63152 : 247,
				63153 : 248,
				63154 : 249,
				63155 : 250,
				63156 : 251,
				63157 : 252,
				63158 : 253,
				63159 : 254,
				63163 : 255,
				63164 : 256,
				63049 : 257,
				63050 : 258,
				63051 : 259,
				63052 : 260,
				63053 : 261,
				63166 : 262,
				63167 : 263,
				63168 : 264,
				63055 : 265,
				63056 : 266,
				63057 : 267,
				63058 : 268,
				63059 : 269,
				63169 : 270,
				63170 : 271,
				63171 : 272,
				63172 : 273,
				63173 : 274,
				63174 : 275,
				63175 : 276,
				63176 : 277,
				63177 : 278,
				63178 : 279,
				63179 : 280,
				63180 : 281,
				63181 : 282,
				63182 : 283,
				63183 : 284,
				63184 : 285,
				63185 : 286,
				63186 : 287,
				63195 : 288,
				63196 : 289,
				63197 : 290,
				63198 : 291,
				63199 : 292,
				63200 : 293,
				63201 : 294,
				63202 : 295,
				63203 : 296,
				63204 : 297,
				63380 : 298,
				63381 : 299,
				63205 : 300,
				63206 : 301,
				63207 : 302,
				63208 : 303,
				63209 : 304,
				63413 : 305,
				63414 : 306,
				63415 : 307,
				63416 : 308,
				63417 : 309,
				63418 : 310,
				63419 : 311,
				63420 : 312,
				63421 : 313,
				63422 : 314,
				63423 : 315,
				63424 : 316,
				63425 : 317,
				63426 : 318,
				63427 : 319,
				63428 : 320,
				63429 : 321,
				63430 : 322,
				63431 : 323,
				63432 : 324,
				63433 : 325,
				63434 : 326,
				63435 : 327,
				63436 : 328,
				63437 : 329,
				63438 : 330,
				63439 : 331,
				63440 : 332,
				63441 : 333,
				63461 : 334,
				63462 : 335,
				63463 : 336,
				63464 : 337,
				63465 : 338,
				63466 : 339,
				63467 : 340,
				63468 : 341,
				63469 : 342,
				63470 : 343,
				63471 : 344,
				63472 : 345,
				63473 : 346,
				63474 : 347,
				63475 : 348,
				63476 : 349,
				63477 : 350,
				63478 : 351,
				63479 : 352,
				63480 : 353,
				63481 : 354,
				63482 : 355,
				63483 : 356,
				63484 : 357,
				62272 : 358,
				62273 : 359,
				62274 : 360,
				62275 : 361,
				62276 : 362,
				62277 : 363,
				62278 : 364,
				62279 : 365,
				62280 : 366,
				62281 : 367,
				62282 : 368,
				62283 : 369,
				62284 : 370,
				62285 : 371,
				62286 : 372,
				62287 : 373,
				62288 : 374,
				62289 : 375,
				62290 : 376,
				62291 : 377,
				62292 : 378,
				62293 : 379,
				62294 : 380,
				62295 : 381,
				62296 : 382,
				62297 : 383,
				62298 : 384,
				62299 : 385,
				62300 : 386,
				62301 : 387,
				62302 : 388,
				62303 : 389,
				62304 : 390,
				62305 : 391,
				62306 : 392,
				62307 : 393,
				62308 : 394,
				62309 : 395,
				62310 : 396,
				62311 : 397,
				62312 : 398,
				62313 : 399,
				62314 : 400,
				62315 : 401,
				62316 : 402,
				62317 : 403,
				62318 : 404,
				62319 : 405,
				62320 : 406,
				62321 : 407,
				62322 : 408,
				62323 : 409,
				62324 : 410,
				62325 : 411,
				62326 : 412,
				62327 : 413,
				62328 : 414,
				62329 : 415,
				62330 : 416,
				62331 : 417,
				62332 : 418,
				62333 : 419,
				62334 : 420,
				62336 : 421,
				62337 : 422,
				62338 : 423,
				62339 : 424,
				62340 : 425,
				62341 : 426,
				62342 : 427,
				62343 : 428,
				62344 : 429,
				62345 : 430,
				62346 : 431,
				62347 : 432,
				62348 : 433,
				62349 : 434,
				62350 : 435,
				62351 : 436,
				62352 : 437,
				62353 : 438,
				62354 : 439,
				62355 : 440,
				62356 : 441,
				62357 : 442,
				62358 : 443,
				62359 : 444,
				62360 : 445,
				62361 : 446,
				62362 : 447,
				62363 : 448,
				62364 : 449,
				62365 : 450,
				62366 : 451,
				62367 : 452,
				62368 : 453,
				62369 : 454,
				62370 : 455,
				62371 : 456,
				62372 : 457,
				62373 : 458,
				62374 : 459,
				62375 : 460,
				62376 : 461,
				62377 : 462,
				62378 : 463,
				62379 : 464,
				62380 : 465,
				62381 : 466,
				62382 : 467,
				62383 : 468,
				62384 : 469,
				62385 : 470,
				62386 : 471,
				62387 : 472,
				62388 : 473,
				62389 : 474,
				62390 : 475,
				62391 : 476,
				62392 : 477,
				62393 : 478,
				62394 : 479,
				62395 : 480,
				62396 : 481,
				62397 : 482,
				62398 : 483,
				62399 : 484,
				62400 : 485,
				62401 : 486,
				62402 : 487,
				62403 : 488,
				62404 : 489,
				62405 : 490,
				62406 : 491,
				62407 : 492,
				62408 : 493,
				62409 : 494,
				62410 : 495,
				62411 : 496,
				62412 : 497,
				62413 : 498,
				62414 : 499,
				63442 : 500,
				63443 : 501,
				63444 : 502,
				63445 : 503,
				63446 : 504,
				63447 : 505,
				63448 : 506,
				63449 : 507,
				63450 : 508,
				63451 : 509,
				63452 : 510,
				63453 : 511,
				63454 : 512,
				63455 : 513,
				63456 : 514,
				63457 : 515,
				63458 : 516,
				63459 : 517,
				63460 : 518,
				62415 : 700,
				62416 : 701,
				62417 : 702,
				62418 : 703,
				62419 : 704,
				62420 : 705,
				62421 : 706,
				62422 : 707,
				62423 : 708,
				62424 : 709,
				62425 : 710,
				62426 : 711,
				62427 : 712,
				62428 : 713,
				62429 : 714,
				62430 : 715,
				62431 : 716,
				62432 : 717,
				62433 : 718,
				62434 : 719,
				62435 : 720,
				62436 : 721,
				62437 : 722,
				62438 : 723,
				62439 : 724,
				62440 : 725,
				62441 : 726,
				62442 : 727,
				62443 : 728,
				62444 : 729,
				62445 : 730,
				62446 : 731,
				62447 : 732,
				62448 : 733,
				62449 : 734,
				62450 : 735,
				62451 : 736,
				62452 : 737,
				62453 : 738,
				62454 : 739,
				62455 : 740,
				62456 : 741,
				62457 : 742,
				62458 : 743,
				62459 : 744,
				62460 : 745,
				62528 : 746,
				62529 : 747,
				62530 : 748,
				62531 : 749,
				62532 : 750,
				62533 : 751,
				62534 : 752,
				62535 : 753,
				62536 : 754,
				62537 : 755,
				62538 : 756,
				62539 : 757,
				62540 : 758,
				62541 : 759,
				62542 : 760,
				62543 : 761,
				62544 : 762,
				62545 : 763,
				62546 : 764,
				62547 : 765,
				62548 : 766,
				62549 : 767,
				62550 : 768,
				62551 : 769,
				62552 : 770,
				62553 : 771,
				62554 : 772,
				62555 : 773,
				62556 : 774,
				62557 : 775,
				62558 : 776,
				62559 : 777,
				62560 : 778,
				62561 : 779,
				62562 : 780,
				62563 : 781,
				62564 : 782,
				62565 : 783,
				62566 : 784,
				62567 : 785,
				62568 : 786,
				62569 : 787,
				62570 : 788,
				62571 : 789,
				62572 : 790,
				62573 : 791,
				62574 : 792,
				62575 : 793,
				62576 : 794,
				62577 : 795,
				62578 : 796,
				62579 : 797,
				62580 : 798,
				62581 : 799,
				62582 : 800,
				62583 : 801,
				62584 : 802,
				62585 : 803,
				62586 : 804,
				62587 : 805,
				62588 : 806,
				62589 : 807,
				62590 : 808,
				62592 : 809,
				62593 : 810,
				62594 : 811,
				62595 : 812,
				62596 : 813,
				62597 : 814,
				62598 : 815,
				62599 : 816,
				62600 : 817,
				62601 : 818,
				62602 : 819,
				62603 : 820,
				62604 : 821,
				62605 : 822
			};
		}
		return this.icomMap;
	},

	getIconMapUnicode : function() {
		if (!this.iconMap) {
			this.icomMap = {
				58472 : 75,
				58473 : 190,
				58474 : 99,
				58475 : 218,
				58476 : 76,
				58477 : 223,
				58478 : 229,
				58479 : 230,
				58480 : 246,
				58481 : 257,
				58482 : 258,
				58483 : 259,
				58484 : 260,
				58485 : 261,
				58486 : 77,
				58487 : 265,
				58488 : 266,
				58489 : 267,
				58490 : 268,
				58491 : 269,
				58492 : 58,
				58493 : 176,
				58494 : 177,
				58495 : 178,
				58496 : 179,
				58497 : 1,
				58498 : 2,
				58499 : 3,
				58500 : 98,
				58501 : 191,
				58502 : 15,
				58503 : 16,
				58504 : 44,
				58505 : 47,
				58506 : 60,
				58507 : 69,
				58508 : 95,
				58509 : 107,
				58510 : 167,
				58511 : 192,
				58512 : 193,
				58513 : 194,
				58514 : 195,
				58515 : 196,
				58516 : 197,
				58517 : 198,
				58518 : 199,
				58519 : 200,
				58520 : 201,
				58521 : 202,
				58522 : 203,
				58523 : 204,
				58524 : 83,
				58525 : 91,
				58526 : 106,
				58527 : 122,
				58528 : 143,
				58529 : 149,
				58530 : 158,
				58531 : 205,
				58532 : 206,
				58533 : 207,
				58534 : 208,
				58535 : 209,
				58536 : 210,
				58537 : 211,
				58538 : 212,
				58539 : 112,
				58540 : 146,
				58541 : 156,
				58542 : 215,
				58543 : 216,
				58544 : 217,
				58545 : 125,
				58546 : 148,
				58547 : 168,
				58548 : 169,
				58549 : 172,
				58550 : 219,
				58551 : 220,
				58552 : 221,
				58553 : 222,
				58554 : 45,
				58555 : 96,
				58556 : 224,
				58557 : 225,
				58558 : 226,
				58559 : 227,
				58560 : 228,
				58561 : 12,
				58562 : 52,
				58563 : 65,
				58564 : 160,
				58565 : 231,
				58566 : 232,
				58567 : 233,
				58568 : 170,
				58569 : 234,
				58570 : 235,
				58571 : 236,
				58572 : 237,
				58573 : 238,
				58574 : 133,
				58575 : 144,
				58576 : 239,
				58577 : 240,
				58578 : 241,
				58579 : 242,
				58580 : 243,
				58581 : 244,
				58582 : 245,
				58583 : 247,
				58584 : 248,
				58585 : 249,
				58586 : 250,
				58587 : 251,
				58588 : 252,
				58589 : 253,
				58590 : 254,
				58591 : 74,
				58592 : 78,
				58593 : 134,
				58594 : 255,
				58595 : 256,
				58596 : 113,
				58597 : 262,
				58598 : 263,
				58599 : 264,
				58600 : 270,
				58601 : 271,
				58602 : 272,
				58603 : 273,
				58604 : 274,
				58605 : 275,
				58606 : 276,
				58607 : 277,
				58608 : 278,
				58609 : 279,
				58610 : 280,
				58611 : 281,
				58612 : 282,
				58613 : 283,
				58614 : 284,
				58615 : 285,
				58616 : 286,
				58617 : 287,
				58618 : 50,
				58619 : 68,
				58620 : 80,
				58621 : 86,
				58622 : 116,
				58623 : 140,
				58624 : 141,
				58625 : 163,
				58626 : 288,
				58627 : 289,
				58628 : 290,
				58629 : 291,
				58630 : 292,
				58631 : 293,
				58632 : 294,
				58633 : 295,
				58634 : 296,
				58635 : 297,
				58636 : 300,
				58637 : 301,
				58638 : 302,
				58639 : 303,
				58640 : 304,
				58641 : 13,
				58642 : 48,
				58643 : 53,
				58644 : 72,
				58645 : 94,
				58646 : 104,
				58647 : 110,
				58648 : 119,
				58649 : 120,
				58650 : 124,
				58651 : 129,
				58652 : 138,
				58653 : 145,
				58654 : 155,
				58655 : 165,
				58656 : 166,
				58657 : 108,
				58658 : 180,
				58659 : 181,
				58660 : 182,
				58661 : 183,
				58662 : 184,
				58663 : 185,
				58664 : 186,
				58665 : 187,
				58666 : 188,
				58667 : 189,
				58668 : 4,
				58669 : 5,
				58670 : 6,
				58671 : 7,
				58672 : 8,
				58673 : 9,
				58674 : 10,
				58675 : 11,
				58676 : 17,
				58677 : 18,
				58678 : 19,
				58679 : 20,
				58680 : 21,
				58681 : 22,
				58682 : 23,
				58683 : 24,
				58684 : 26,
				58685 : 27,
				58686 : 28,
				58687 : 29,
				58688 : 30,
				58689 : 31,
				58690 : 32,
				58691 : 33,
				58692 : 34,
				58693 : 35,
				58694 : 36,
				58695 : 37,
				58696 : 38,
				58697 : 39,
				58698 : 40,
				58699 : 41,
				58700 : 42,
				58701 : 43,
				58702 : 54,
				58703 : 55,
				58704 : 61,
				58705 : 62,
				58706 : 63,
				58707 : 64,
				58708 : 66,
				58709 : 70,
				58710 : 71,
				58711 : 73,
				58712 : 81,
				58713 : 82,
				58714 : 88,
				58715 : 89,
				58716 : 117,
				58717 : 118,
				58718 : 132,
				58719 : 136,
				58720 : 49,
				58721 : 56,
				58722 : 59,
				58723 : 67,
				58724 : 92,
				58725 : 97,
				58726 : 100,
				58727 : 101,
				58728 : 102,
				58729 : 103,
				58730 : 105,
				58731 : 121,
				58732 : 131,
				58733 : 137,
				58734 : 142,
				58735 : 147,
				58736 : 157,
				58737 : 213,
				58738 : 214,
				58739 : 90,
				58740 : 127,
				58741 : 128,
				58742 : 159,
				58743 : 298,
				58744 : 299,
				58745 : 14,
				58746 : 25,
				58747 : 57,
				58748 : 87,
				58749 : 109,
				58750 : 111,
				58751 : 114,
				58752 : 115,
				58753 : 123,
				58754 : 126,
				58755 : 130,
				58756 : 135,
				58757 : 139,
				58758 : 150,
				58759 : 152,
				58760 : 161,
				58761 : 162,
				58762 : 164,
				58763 : 171,
				58764 : 173,
				58765 : 174,
				58766 : 175,
				58767 : 79,
				58768 : 84,
				58769 : 151,
				58770 : 153,
				58771 : 154,
				58772 : 46,
				58773 : 51,
				58774 : 85,
				58775 : 93,
				58776 : 305,
				58777 : 306,
				58778 : 307,
				58779 : 308,
				58780 : 309,
				58781 : 310,
				58782 : 311,
				58783 : 312,
				58784 : 313,
				58785 : 314,
				58786 : 315,
				58787 : 316,
				58788 : 317,
				58789 : 318,
				58790 : 319,
				58791 : 320,
				58792 : 321,
				58793 : 322,
				58794 : 323,
				58795 : 324,
				58796 : 325,
				58797 : 326,
				58798 : 327,
				58799 : 328,
				58800 : 329,
				58801 : 330,
				58802 : 331,
				58803 : 332,
				58804 : 333,
				58805 : 334,
				58806 : 335,
				58807 : 336,
				58808 : 337,
				58809 : 338,
				58810 : 339,
				58811 : 340,
				58812 : 341,
				58813 : 342,
				58814 : 343,
				58815 : 344,
				58816 : 345,
				58817 : 346,
				58818 : 347,
				58819 : 348,
				58820 : 349,
				58821 : 350,
				58822 : 351,
				58823 : 352,
				58824 : 353,
				58825 : 354,
				58826 : 355,
				58827 : 356,
				58828 : 357,
				58829 : 358,
				58830 : 359,
				58831 : 360,
				58832 : 361,
				58833 : 362,
				58834 : 363,
				58835 : 364,
				58836 : 365,
				58837 : 366,
				58838 : 367,
				58839 : 368,
				58840 : 369,
				58841 : 370,
				58842 : 371,
				58843 : 372,
				58844 : 373,
				58845 : 374,
				58846 : 375,
				58847 : 376,
				60032 : 377,
				60033 : 378,
				60034 : 379,
				60035 : 380,
				60036 : 381,
				60037 : 382,
				60038 : 383,
				60039 : 384,
				60040 : 385,
				60041 : 386,
				60042 : 387,
				60043 : 388,
				60044 : 389,
				60045 : 390,
				60046 : 391,
				60047 : 392,
				60048 : 393,
				60049 : 394,
				60050 : 395,
				60051 : 396,
				60052 : 397,
				60053 : 398,
				60054 : 399,
				60055 : 400,
				60056 : 401,
				60057 : 402,
				60058 : 403,
				60059 : 404,
				60060 : 405,
				60061 : 406,
				60062 : 407,
				60063 : 408,
				60064 : 409,
				60065 : 410,
				60066 : 411,
				60067 : 412,
				60068 : 413,
				60069 : 414,
				60070 : 415,
				60071 : 416,
				60072 : 417,
				60073 : 418,
				60074 : 419,
				60075 : 420,
				60076 : 421,
				60077 : 422,
				60078 : 423,
				60079 : 424,
				60080 : 425,
				60081 : 426,
				60082 : 427,
				60083 : 428,
				60084 : 429,
				60085 : 430,
				60086 : 431,
				60087 : 432,
				60088 : 433,
				60089 : 434,
				60090 : 435,
				60091 : 436,
				60092 : 437,
				60093 : 438,
				60094 : 439,
				60095 : 440,
				60096 : 441,
				60097 : 442,
				60098 : 443,
				60099 : 444,
				60100 : 445,
				60101 : 446,
				60102 : 447,
				60103 : 448,
				60104 : 449,
				60105 : 450,
				60106 : 451,
				60107 : 452,
				60108 : 453,
				60109 : 454,
				60110 : 455,
				60111 : 456,
				60112 : 457,
				60113 : 458,
				60114 : 459,
				60115 : 460,
				60116 : 461,
				60117 : 462,
				60118 : 463,
				60119 : 464,
				60120 : 465,
				60121 : 466,
				60122 : 467,
				60123 : 468,
				60124 : 469,
				60125 : 470,
				60126 : 471,
				60127 : 472,
				60128 : 473,
				60129 : 474,
				60130 : 475,
				60131 : 476,
				60132 : 477,
				60133 : 478,
				60134 : 479,
				60135 : 480,
				60136 : 481,
				60137 : 482,
				60138 : 483,
				60139 : 484,
				60140 : 485,
				60141 : 486,
				60142 : 487,
				60143 : 488,
				60144 : 489,
				60145 : 490,
				60146 : 491,
				60147 : 492,
				60148 : 493,
				60149 : 494,
				60150 : 495,
				60151 : 496,
				60152 : 497,
				60153 : 498,
				60154 : 499,
				60155 : 500,
				60156 : 501,
				60157 : 502,
				60158 : 503,
				60159 : 504,
				60160 : 505,
				60161 : 506,
				60162 : 507,
				60163 : 508,
				60164 : 509,
				60165 : 510,
				60166 : 511,
				60167 : 512,
				60168 : 513,
				60169 : 514,
				60170 : 515,
				60171 : 516,
				60172 : 517,
				60173 : 518,
				60174 : 700,
				60175 : 701,
				60176 : 702,
				60177 : 703,
				60178 : 704,
				60179 : 705,
				60180 : 706,
				60181 : 707,
				60182 : 708,
				60183 : 709,
				60184 : 710,
				60185 : 711,
				60186 : 712,
				60187 : 713,
				60188 : 714,
				60189 : 715,
				60190 : 716,
				60191 : 717,
				60192 : 718,
				60193 : 719,
				60194 : 720,
				60195 : 721,
				60196 : 722,
				60197 : 723,
				60198 : 724,
				60199 : 725,
				60200 : 726,
				60201 : 727,
				60202 : 728,
				60203 : 729,
				60204 : 730,
				60205 : 731,
				60206 : 732,
				60207 : 733,
				60208 : 734,
				60209 : 735,
				60210 : 736,
				60211 : 737,
				60212 : 738,
				60213 : 739,
				60214 : 740,
				60215 : 741,
				60216 : 742,
				60217 : 743,
				60218 : 744,
				60219 : 745,
				60220 : 746,
				60221 : 747,
				60222 : 748,
				60223 : 749,
				60224 : 750,
				60225 : 751,
				60226 : 752,
				60227 : 753,
				60228 : 754,
				60229 : 755,
				60230 : 756,
				60231 : 757,
				60232 : 758,
				60233 : 759,
				60234 : 760,
				60235 : 761,
				60236 : 762,
				60237 : 763,
				60238 : 764,
				60239 : 765,
				60240 : 766,
				60241 : 767,
				60242 : 768,
				60243 : 769,
				60244 : 770,
				60245 : 771,
				60246 : 772,
				60247 : 773,
				60248 : 774,
				60249 : 775,
				60250 : 776,
				60251 : 777,
				60252 : 778,
				60253 : 779,
				60254 : 780,
				60255 : 781,
				60256 : 782,
				60257 : 783,
				60258 : 784,
				60259 : 785,
				60260 : 786,
				60261 : 787,
				60262 : 788,
				60263 : 789,
				60264 : 790,
				60265 : 791,
				60266 : 792,
				60267 : 793,
				60268 : 794,
				60269 : 795,
				60270 : 796,
				60271 : 797,
				60272 : 798,
				60273 : 799,
				60274 : 800,
				60275 : 801,
				60276 : 802,
				60277 : 803,
				60278 : 804,
				60279 : 805,
				60280 : 806,
				60281 : 807,
				60282 : 808,
				60283 : 809,
				60284 : 810,
				60285 : 811,
				60286 : 812,
				60287 : 813,
				60288 : 814,
				60289 : 815,
				60290 : 816,
				60291 : 817,
				60292 : 818,
				60293 : 819,
				60294 : 820,
				60295 : 821,
				60296 : 822
			};
		}
		return this.icomMap;
	}
};
